from astropy.constants import c,au,G,pc
from astropy.units import year as yr
from astropy.cosmology import WMAP9 as cosmo

Msun = 4.9255e-6
year = (1*yr).cgs.value
Mpc = 1e6*pc.value/c.value
euler_gamma = 0.577215664901532860606512090082
MR_sun = 1.476625061404649406193430731479084713e3
speed_of_light = 299792458.0

